# check if libraries are installed
# check if net-snmp-config script is on the system
SNMPDEVEL=`rpm -q net-snmp-devel`
SNMPUTILS=`rpm -q net-snmp-utils`
if echo ${SNMPDEVEL} | grep -qi 'not' ; then
	SNMPDEVEL=""
fi
if echo ${SNMPUTILS} | grep -qi 'not' ; then
	SNMPUTILS=""
fi
if [ -n "${SNMPDEVEL}" -a -n "${SNMPUTILS}" ]; then
	echo "-lnetsnmpagent -lnetsnmpmibs -lnetsnmphelpers -lnetsnmp -lrpmdb -lrpm -lpopt -lrpmio -lwrap -lnsl "
else
# hopefully ucd-snmp is installed
	echo "-lucdagent -lucdmibs -lsnmp"
fi
